@@ -1,9 +1,9 @@
-/*      $Id: lirc.c,v 1.17 2001/01/06 15:41:31 columbus Exp $      */
+/*      $Id: lirc.c,v 1.22 2003/05/03 18:09:31 columbus Exp $      */
 
 /* Lirc xmms Plugin
    Copyright (c) 1998-1999 Carl van Schaik (carl@leg.uct.ac.za)
 
-   Copyright (C) 2000 Christoph Bartelmus (xmms@bartelmus.de)
+   Copyright (C) 2000-2003 Christoph Bartelmus (xmms@bartelmus.de)
 
    some code was stolen from:
    IRman plugin for xmms by Charles Sielski (stray@teklabs.net)
@@ -49,9 +49,16 @@
 #include <eci.h>
 #endif
 
+#ifdef HAVE_XOSD
+#include "xosd.h"
+#endif
+
 #include "lirc.h"
 
 const char *plugin_name="LIRC Plugin " VERSION;
+const char *default_fontname=
+"-xxl-*-medium-r-semicondensed-*-*-*-75-75-*-*-*-*";
+const char *default_colourname="LawnGreen";
 
 GeneralPlugin lirc_plugin = {
 	NULL,
@@ -78,12 +85,17 @@ gint mute_vol=0;    /* holds volume before mute */
 gint input_tag;
 
 int select_tag=0;
+int sleep_tag=0;
 GtkWidget *select_popup=NULL;
 GtkStyle *popup_style=NULL;
 GList *select_list=NULL;
 GtkWidget *fsd;
 gchar *fontname;
 
+#ifdef HAVE_XOSD
+xosd *osd=NULL;
+#endif
+
 struct select_info
 {
 	char *s;
@@ -96,24 +108,38 @@ struct title_info
 	int pos;
 };
 
+gint sleep_timeout(GtkWidget **select_popup)
+{
+	xmms_remote_pause(lirc_plugin.xmms_session);
+	sleep_tag=0;
+	return(FALSE);
+}
+
 #ifdef HAVE_VIAVOICE_TTS
 ECIHand eciHandle;
 
 void say_title(void)
 {
-	gchar *title;
-	gint pos;
-					
+	gchar *title,*clear="_~";
+	gint pos,i;
+	
 	pos=xmms_remote_get_playlist_pos(lirc_plugin.xmms_session);
 	if(pos>=0)
-	{	
+	{
+		
 		title=xmms_remote_get_playlist_title(lirc_plugin.
 						     xmms_session,pos);
+		for(i=0;i<strlen(title);i++)
+		{
+			if(strchr(clear,title[i])) title[i]=' ';
+		}
 		eciAddText(eciHandle,title);
 		usleep(500000);
 		eciSynthesize(eciHandle);
 		eciSynchronize(eciHandle);
 		usleep(500000);
+		
+		g_free(title);
 	}
 }
 
@@ -214,39 +240,37 @@ gchar *get_title_postfix(gchar *title)
 		if(!isalnum(*match))
 		{
 			match++;
+			continue;
+		}
+		if(strchr(si->s,toupper(*match))!=NULL)
+		{
+			skip=si->common;
+			while(skip>0)
+			{
+				if(isalnum(*match))
+				{
+					skip--;
+				}
+				match++;
+				if(!*match)
+				{
+					break;
+				}
+			}
+			scan_list=g_list_next(scan_list);
+			if(scan_list==NULL)
+			{
+				while(!isalnum(*match) && *match)
+				{
+					match++;
+				}
+				return(match);
+			}
+			si=(struct select_info *) (scan_list->data);
 		}
 		else
 		{
-			if(strchr(si->s,toupper(*match))!=NULL)
-			{
-				skip=si->common;
-				while(skip>0)
-				{
-					if(isalnum(*match))
-					{
-						skip--;
-					}
-					match++;
-					if(!*match)
-					{
-						break;
-					}
-				}
-				scan_list=g_list_next(scan_list);
-				if(scan_list==NULL)
-				{
-					while(!isalnum(*match) && *match)
-					{
-						match++;
-					}
-					return(match);
-				}
-				si=(struct select_info *) (scan_list->data);
-			}
-			else
-			{
-				break;
-			}
+			break;
 		}
 	}
 	return(NULL);
@@ -254,6 +278,15 @@ gchar *get_title_postfix(gchar *title)
 
 #define SELECT_BORDER 8
 
+#ifdef HAVE_XOSD
+gint remove_select_popup(xosd **osd)
+{
+	xosd_uninit(*osd);
+	*osd=NULL;
+	select_tag=0;
+	return(FALSE);
+}
+#else
 gint remove_select_popup(GtkWidget **select_popup)
 {
 	gtk_widget_destroy(*select_popup);
@@ -261,24 +294,31 @@ gint remove_select_popup(GtkWidget **select_popup)
 	select_tag=0;
 	return(FALSE);
 }
+#endif
 
 void select_title(char *s,int num)
 {
-	GdkFont *font;
-	GtkStyle *style;
-	int gap,baseline_skip;
-	int width,height,x,y;
 	GList *new_list,*scan_list;
 	GList *title_list=NULL;
 	int length,i;
 	gchar *title;
 	struct select_info *si;
 	struct title_info *ti;
+#ifndef HAVE_XOSD
+	GdkFont *font;
+	GtkStyle *style;
+	int gap,baseline_skip;
+	int width,height,x,y;
+#endif
 	
 	if(select_tag!=0)
 	{
 		gtk_timeout_remove(select_tag);
+#ifdef HAVE_XOSD
+		remove_select_popup(&osd);
+#else
 		remove_select_popup(&select_popup);
+#endif
 	}
 	if(s!=NULL)
 	{
@@ -325,6 +365,7 @@ void select_title(char *s,int num)
 			if(ti->s==NULL)
 			{
 				free(ti);
+				g_free(title);
 				break;
 			}
 			ti->pos=i;
@@ -333,11 +374,27 @@ void select_title(char *s,int num)
 			{
 				free(ti->s);
 				free(ti);
+				g_free(title);
 				break;
 			}
 			title_list=new_list;
 		}
+		g_free(title);
 	}
+#ifdef HAVE_XOSD
+	if(fontname!=NULL)
+	{
+		osd=xosd_init(fontname,(char *) default_colourname,
+			      3,XOSD_top,0,1);
+	}
+	else
+	{
+		osd=xosd_init((char *) default_fontname,
+			      (char *) default_colourname,3,XOSD_top,0,1);
+	}
+	
+	if(osd==NULL) return;
+#else
 	select_popup=gtk_window_new(GTK_WINDOW_POPUP);
 	if(popup_style)
 	{
@@ -347,12 +404,11 @@ void select_title(char *s,int num)
 		}
 		else
 		{
-			font=gdk_font_load
-				("-xxl-*-medium-r-semicondensed-*-*-*-75-75-*-*-*-*");
+			font=gdk_font_load(default_fontname);
 		}
 		if(font) popup_style->font=font;
 		gtk_widget_set_style(select_popup,popup_style);
-	}	
+	}
 	gtk_widget_set_app_paintable(select_popup,TRUE);
 	gtk_window_set_policy(GTK_WINDOW(select_popup),FALSE,FALSE,TRUE);
 	gtk_widget_set_name(select_popup,"LIRC select list");
@@ -361,11 +417,15 @@ void select_title(char *s,int num)
 	gap=(font->ascent+font->descent)/4;
 	if(gap<2) gap=2;
 	baseline_skip=font->ascent+font->descent+gap;
+#endif
 	
 	if(title_list==NULL)
 	{
 		gchar *text="No match.";
 		
+#ifdef HAVE_XOSD
+		xosd_display(osd, 0, XOSD_string, text);
+#else
 		x=gdk_string_width(font,text);
 		width=SELECT_BORDER*2+x;
 		height=SELECT_BORDER*2+baseline_skip-gap;
@@ -387,7 +447,7 @@ void select_title(char *s,int num)
 				 SELECT_BORDER,
 				 SELECT_BORDER+font->ascent,
 				 "No match.");
-		
+#endif
 		clear_select_list(&select_list);
 	}
 	else
@@ -477,6 +537,11 @@ void select_title(char *s,int num)
 			help=postfix[0];
 			postfix[0]=0;
 			
+#ifdef HAVE_XOSD
+			snprintf(buffer,20,"%02d: %s %s",count,title,postfix);
+                        if(count<20) 
+                        xosd_display(osd,count-1,XOSD_string,buffer);
+#else
 			snprintf(buffer,20,"%2d:",count);
 			if(pass==1)
 			{
@@ -489,6 +554,7 @@ void select_title(char *s,int num)
 				x=gdk_string_width(select_popup->style->font,
 						   postfix);
 				if(x>x3) x3=x;
+
 			}
 			else
 			{
@@ -519,7 +585,9 @@ void select_title(char *s,int num)
 						 postfix);
 				
 			}
+#endif
 			scan_list=g_list_next(scan_list);
+#ifndef HAVE_XOSD
 			if(scan_list==NULL)
 			{
 				if(pass==1)
@@ -548,6 +616,9 @@ void select_title(char *s,int num)
 				}
 				pass++;
 			}
+#else
+			pass=0;
+#endif
 		}
 		if(count==1)
 		{
@@ -560,8 +631,13 @@ void select_title(char *s,int num)
 		clear_title_list(&title_list);
 		
 	}
+#ifdef HAVE_XOSD
+	select_tag=gtk_timeout_add(3000,(GtkFunction) remove_select_popup,
+				   (gpointer) &osd);
+#else
 	select_tag=gtk_timeout_add(3000,(GtkFunction) remove_select_popup,
 				   (gpointer) &select_popup);
+#endif
 }
 
 void init(void)
@@ -659,14 +735,37 @@ void lirc_input_callback(gpointer data,gint source,
 			}
 			else if(strcasecmp("PLAYPAUSE",c) == 0)
 			{
-				if(xmms_remote_is_playing(lirc_plugin.
-							  xmms_session))
+				if(xmms_remote_is_playing
+				   (lirc_plugin.xmms_session)
+				   && !xmms_remote_is_paused
+				   (lirc_plugin.xmms_session))
 					xmms_remote_pause
 						(lirc_plugin.xmms_session);
 				else
 					xmms_remote_play
 						(lirc_plugin.xmms_session);
 			}
+			else if(strncasecmp("SLEEP",c,5)==0)
+			{
+                                ptr=c+5;
+                                while(isspace(*ptr)) ptr++;
+				n=atoi(ptr);
+				
+				if(n<=0) n=5;
+				if(sleep_tag==0)
+				{
+					sleep_tag=gtk_timeout_add
+						(n*60*1000,
+						 (GtkFunction) sleep_timeout,
+						 NULL);
+				}
+				else
+				{
+					gtk_timeout_remove(sleep_tag);
+					sleep_tag=0;
+				}
+				
+			}
 			else if(strncasecmp("NEXT",c,4)==0)
 			{
                                 ptr=c+4;
@@ -970,12 +1069,59 @@ void lirc_input_callback(gpointer data,gint source,
 					(lirc_plugin.xmms_session);
 				list.prev=list.next=NULL;
 				list.data=c+13;
+#ifdef XMMS_REMOTE_PLAYLIST_ADD_BROKEN
+				xmms_remote_play_files
+					(lirc_plugin.xmms_session,&list);
+#else
 				xmms_remote_playlist_add
 					(lirc_plugin.xmms_session,&list);
+#endif
 				/* This is to refresh window content */
 				xmms_remote_pl_win_toggle
 					(lirc_plugin.xmms_session,pl_visible);
                         }
+                        else if(strncasecmp("PLAYLIST_SET ",c,13)==0)
+                        {
+                                gboolean pl_visible;
+                                GList list;
+                                int b;
+				
+                                pl_visible=xmms_remote_is_pl_win
+                                        (lirc_plugin.xmms_session);
+                                list.prev=list.next=NULL;
+                                list.data=c+13;
+				
+                                b=xmms_remote_is_playing
+                                        (lirc_plugin.xmms_session);
+				
+                                xmms_remote_stop(lirc_plugin.xmms_session);
+                                xmms_remote_playlist_clear
+                                        (lirc_plugin.xmms_session);
+                                xmms_remote_playlist_add
+                                        (lirc_plugin.xmms_session,&list);
+                                if(b)
+				{
+                                        xmms_remote_play
+                                                (lirc_plugin.xmms_session);
+                                }
+                                /* This is to refresh window content */
+                                xmms_remote_pl_win_toggle
+                                        (lirc_plugin.xmms_session,pl_visible);
+                        }
+			else if(strcasecmp("PLAYLIST_REMOVE",c)==0)
+			{
+				gboolean pl_visible;
+
+				pl_visible=xmms_remote_is_pl_win
+					(lirc_plugin.xmms_session);
+				xmms_remote_playlist_delete
+					(lirc_plugin.xmms_session,
+					 xmms_remote_get_playlist_pos
+					 (lirc_plugin.xmms_session));
+				/* This is to refresh window content */
+				xmms_remote_pl_win_toggle
+					(lirc_plugin.xmms_session,pl_visible);
+			}
 #ifdef HAVE_VIAVOICE_TTS
 			else if(strcasecmp("SAYTITLE",c)==0)
 			{
